@@ -1,12 +1,14 @@
 #!/usr/bin/env python
 
-
 from csvmedkit import agate
-from csvmedkit.cmkutil import CmkUtil
+from csvmedkit.cmk.cmkutil import CmkUtil, UniformReader
 from shutil import get_terminal_size
 import sys
 import textwrap
-from typing import NoReturn as typeNoReturn
+from typing import (
+    Callable as typeCallable,
+    Optional as typeOptional,
+)
 
 DEFAULT_EOR_MARKER = "~"
 DEFAULT_MAX_LENGTH = 50
@@ -26,7 +28,7 @@ FLAT_COL_PADDING = 4
 FLAT_COL_WIDTH = len("field") + FLAT_COL_PADDING  # e.g. '| field |' and '| recid |'
 
 
-class CSVFlatten(CmkUtil):
+class CSVFlatten(UniformReader, CmkUtil):
     description = """Prints flattened records, such that each row represents a record's fieldname and corresponding value,
                      similar to transposing a record in spreadsheet format"""
 
@@ -75,28 +77,82 @@ class CSVFlatten(CmkUtil):
             help="""Print output in tabular format instead of CSV""",
         )
 
+    @property
+    def chunkpattern(self) -> typeCallable:
+        """
+        expects self.max_field_length to be set
+        """
+        # if max_field_length is specified, then the pattern for chunking uses it
+        if self.max_field_length:
+            _cp = lambda txt: textwrap.wrap(txt, width=self.max_field_length)
+        else:  # we just split newlines
+            _cp = lambda txt: txt.splitlines()
+        return _cp
+
+    @property
+    def end_of_record_marker(self) -> typeOptional[str]:
+        """
+        preconditions:
+            - self.max_column_name_length
+            - self.rec_ids_mode
+        """
+        marker: typeOptional[str]
+
+        _eor = self.args.end_of_record_marker
+        if _eor == "none" or _eor == "":
+            marker = None
+        elif _eor:  # use default
+            marker = _eor
+        else:
+            # disable by default, if we're in rowid mode
+            if self.rec_ids_mode:
+                marker = None
+            else:
+                marker = "".join(
+                    DEFAULT_EOR_MARKER for i in range(self.max_column_name_length)
+                )
+        return marker
+
+    @property
+    def label_chunks_mode(self):
+        return self.args.label_chunks_mode
+
+    @property
+    def rec_ids_mode(self):
+        return self.args.rec_ids_mode
+
+    @property
+    def prettify(self):
+        return self.args.prettify
+
+    def read_input(self):
+        self._rows = agate.csv.reader(self.skip_lines(), **self.reader_kwargs)
+        self._column_names = next(self._rows)
+        self._read_input_done = True
+
     def main(self):
         if self.additional_input_expected():
             self.argparser.error("You must provide an input file or piped data.")
 
-        # the following args are referenced in self.reader_kwargs and used in get_rows_and_column_names_and_column_ids
-        rows = self.text_csv_reader()
-        column_names = next(rows)
-        self.rec_ids_mode = self.args.rec_ids_mode
+        # UniformReader (DRY later)
+        self.read_input()
+        if self.is_empty:
+            return
+        # /UniformReader
+
+        outrows: list
+        outtable: agate.Table
+
         self.output_flat_column_names = (
             FLAT_COLUMN_NAMES
             if not self.rec_ids_mode
             else (REC_ID_COLUMN_NAME,) + FLAT_COLUMN_NAMES
         )
 
-        self.max_column_name_length = (
-            max(len(c) for c in (column_names + list(self.output_flat_column_names)))
-            if column_names
-            else 0  # for empty file case
+        self.max_column_name_length = max(
+            len(c) for c in (self.i_column_names + list(self.output_flat_column_names))
         )
 
-        self.prettify = True if self.args.prettify else False
-
         if (
             self.args.max_field_length or self.args.max_field_length == 0
         ):  # 0 is considered to be infinite/no-wrap
@@ -115,30 +171,9 @@ class CSVFlatten(CmkUtil):
         else:
             self.max_field_length = None
 
-        _eor = self.args.end_of_record_marker
-        if _eor == "none" or _eor == "":
-            self.end_of_record_marker = None
-        elif _eor:  # use default
-            self.end_of_record_marker = _eor
-        else:
-            # disable by default, if we're in rowid mode
-            if self.rec_ids_mode:
-                self.end_of_record_marker = None
-            else:
-                self.end_of_record_marker = "".join(
-                    DEFAULT_EOR_MARKER for i in range(self.max_column_name_length)
-                )
-
-        # if max_field_length is specified, then the pattern for chunking uses it
-        if self.max_field_length:
-            self.chunkpattern = lambda txt: textwrap.wrap(
-                txt, width=self.max_field_length
-            )
-        else:  # we just split newlines
-            self.chunkpattern = lambda txt: txt.splitlines()
-
         outrows = []
-        for row_idx, row in enumerate(rows):
+        for row_idx, row in enumerate(self.i_rows):
+            # TODO: wtf is this spagehtti:
             o_row = (
                 [
                     row_idx,
@@ -152,7 +187,7 @@ class CSVFlatten(CmkUtil):
                 eor_row = [None] if self.rec_ids_mode else []
                 outrows.append(eor_row + [self.end_of_record_marker, None])
 
-            for col_idx, colname in enumerate(column_names):
+            for col_idx, colname in enumerate(self.i_column_names):
                 # value_lines = row[col_idx].strip().splitlines()
                 # for line in value_lines:
                 txt = row[col_idx].strip()
@@ -163,12 +198,12 @@ class CSVFlatten(CmkUtil):
                     else:
                         fieldname = (
                             f"{colname}__{chunk_idx}"
-                            if self.args.label_chunks_mode is True
+                            if self.label_chunks_mode is True
                             else None
                         )
                     outrows.append(o_row + [fieldname, chunk])
 
-        if self.args.prettify:
+        if self.prettify:
             outtable = agate.Table(
                 outrows,
                 column_names=self.output_flat_column_names,
